"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.behaviorpack_check_command_blockstates = exports.behaviorpack_check_blockstates = void 0;
const bc_minecraft_bedrock_types_1 = require("bc-minecraft-bedrock-types");
const types_1 = require("bc-minecraft-bedrock-types/lib/src/types");
const bc_minecraft_bedrock_vanilla_data_1 = require("bc-minecraft-bedrock-vanilla-data");
const Molang_1 = require("bc-minecraft-molang/lib/src/Molang");
const Types_1 = require("../../../Types");
const Definitions_1 = require("../../Definitions");
/** Checks if the blocks exists in the project or in vanilla, if not then a bug is reported
 * @param id
 * @param diagnoser
 * @returns
 */
function behaviorpack_check_blockstates(blockDescriptor, diagnoser) {
    //If the block has no states then skip
    if (!blockDescriptor.text.includes("["))
        return;
    //Parses states
    const blockData = bc_minecraft_bedrock_types_1.Minecraft.Block.fromBlockDescriptor(blockDescriptor.text);
    check_block_definition(blockData, blockDescriptor, diagnoser);
}
exports.behaviorpack_check_blockstates = behaviorpack_check_blockstates;
/**
 *
 * @param blockId
 * @param states
 * @param diagnoser
 */
function behaviorpack_check_command_blockstates(blockId, states, diagnoser) {
    const blockData = {
        id: blockId.text,
        location: types_1.Location.empty(),
        states: [],
    };
    // Is state properly formatted?
    if (states.text.startsWith('[') && states.text.endsWith(']')) {
        const value = states.text.substring(1, states.text.length - 1);
        const split = value.split(',');
        // For each state
        for (var I = 0; I < split.length; I++) {
            const item = split[I];
            const state = split[I].split(':');
            // Is state properly defined
            if (state.length == 2) {
                let [property, value] = state;
                // property is a string literal?
                if (property.startsWith('"') && property.endsWith('"')) {
                    property = property.substring(1, property.length - 1);
                    blockData.states.push({ value, property });
                }
                else {
                    diagnoser.add(blockId, `Invalid state: '${property}' in '${item}' on the block state definition: '${states.text}', needs to be a string literal with ""`, Types_1.DiagnosticSeverity.error, "behaviorpack.block.states.invalid");
                }
            }
            else if (state[0] !== "") {
                diagnoser.add(states, `Invalid state: '${item}' in the block command, needs to be a key value pair with :`, Types_1.DiagnosticSeverity.error, "behaviorpack.block.states.invalid");
            }
        }
    }
    else {
        diagnoser.add(states, `Invalid states: '${states.text}' in the block command, needs to be a list with []`, Types_1.DiagnosticSeverity.error, "behaviorpack.block.states.invalid");
    }
    check_block_definition(blockData, states, diagnoser);
}
exports.behaviorpack_check_command_blockstates = behaviorpack_check_command_blockstates;
function check_block_definition(blockDefinition, location, diagnoser) {
    var _a;
    const data = diagnoser.context.getCache();
    const block = (_a = data.BehaviorPacks.blocks.get(blockDefinition.id)) !== null && _a !== void 0 ? _a : vanilla_block(diagnoser, blockDefinition.id);
    //No block found, expecting behaviorpack_check_blockId has been run
    if (!block)
        return;
    if (block.states.length == 0 && blockDefinition.states.length > 0) {
        //Block has not defined states, but states are being used
        diagnoser.add(location, `Block: ${block.id} has no defined states`, Types_1.DiagnosticSeverity.error, "behaviorpack.block.states.missing");
        return;
    }
    for (var I = 0; I < blockDefinition.states.length; I++) {
        const state = blockDefinition.states[I];
        check_state(state, block, location, diagnoser);
    }
}
/**
 *
 * @param blockDescriptor
 * @param state
 * @param data
 * @param diagnoser
 * @returns
 */
function check_state(state, data, location, diagnoser) {
    for (var I = 0; I < data.states.length; I++) {
        const stateData = data.states[I];
        //If found state with the same name
        if (stateData.name === state.property) {
            let actual = state.value;
            const values = stateData.values;
            if (stateData.type === "string") {
                if (actual.startsWith('"') && actual.endsWith('"')) {
                    actual = actual.substring(1, actual.length - 1);
                }
                else {
                    diagnoser.add(location, `Invalid state value: '${state.value}' for state: '${state.property}' in the block definition: '${data.id}', needs to be a string literal with ""`, Types_1.DiagnosticSeverity.error, "behaviorpack.block.states.invalid");
                    return;
                }
            }
            //Check if the state value is valid
            for (let expect of values) {
                // Compare int/bool/string values
                if (expect == actual) {
                    return;
                }
            }
            diagnoser.add(location, `Invalid state value: '${state.value}' for state: '${state.property}' in the block definition: '${data.id}'\nValid values are: ${values.join(",")}`, Types_1.DiagnosticSeverity.error, "behaviorpack.block.states.invalid");
            return;
        }
    }
    //No state matching found
    diagnoser.add(location, `Missing state: '${state.property}' in the block definition: '${data.id}'`, Types_1.DiagnosticSeverity.error, "behaviorpack.block.states.missing");
}
function vanilla_block(diagnoser, blockId) {
    if (!blockId.startsWith("minecraft:")) {
        blockId = `minecraft:${blockId}`;
    }
    const edu = (0, Definitions_1.education_enabled)(diagnoser);
    const block = bc_minecraft_bedrock_vanilla_data_1.MinecraftData.BehaviorPack.getBlock(blockId, edu);
    if (block) {
        const result = {
            id: block.id,
            molang: Molang_1.MolangSet.create(),
            location: bc_minecraft_bedrock_types_1.Types.Location.empty(),
            states: [],
        };
        for (let prop of block.properties) {
            const state = bc_minecraft_bedrock_vanilla_data_1.MinecraftData.BehaviorPack.getBlockState(prop);
            if (state)
                result.states.push(state);
        }
        return result;
    }
    return undefined;
}
//# sourceMappingURL=diagnose.js.map