"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Diagnoser = void 0;
const bc_minecraft_bedrock_project_1 = require("bc-minecraft-bedrock-project");
const Severity_1 = require("./Severity");
const Format_1 = require("../Diagnostics/Format");
const Minecraft_1 = require("../Diagnostics/Minecraft");
const path = require("path");
const BehaviorPack_1 = require("../Diagnostics/BehaviorPack");
const ResourcePack_1 = require("../Diagnostics/ResourcePack");
const SkinPack_1 = require("../Diagnostics/SkinPack/SkinPack");
const WorldPack_1 = require("../Diagnostics/WorldPack/WorldPack");
/**The object that is responsible for diagnosing minecraft bedrock files*/
class Diagnoser {
    /**Create a new instance of Diagnoser
     * @param context The context needed to perform diagnostics*/
    constructor(context) {
        this.context = context;
    }
    /** Process and diagnoses the given document
     * @param doc The textdocument to process or the uri to the document
     * @returns `true` or `false` if the diagnostics was successfully*/
    process(doc) {
        if (typeof doc === "string") {
            const temp = this.context.getDocument(doc);
            if (!temp)
                return false;
            doc = temp;
        }
        const pack = this.context.getCache().get(doc);
        if (!pack)
            return false;
        //Check if diagnostics was disabled
        const ext = path.extname(doc.uri);
        //Check if diagnostics for this file type is disabled
        if (pack.context.attributes["diagnostic" + ext] === "false")
            return false;
        const diagnoser = this.context.getDiagnoser(doc, pack.context);
        if (!diagnoser)
            return false;
        diagnoser.document = doc;
        let out = false;
        try {
            //diagnose path
            (0, Format_1.format_diagnose_path)(pack, doc.uri, diagnoser);
            //Language file?
            if (doc.uri.endsWith(".lang")) {
                (0, Minecraft_1.minecraft_language_diagnose)(diagnoser);
            }
            else {
                //Check per pack
                switch (pack.type) {
                    case bc_minecraft_bedrock_project_1.PackType.behavior_pack:
                        out = BehaviorPack_1.BehaviorPack.Process(diagnoser);
                        break;
                    case bc_minecraft_bedrock_project_1.PackType.resource_pack:
                        out = ResourcePack_1.ResourcePack.Process(diagnoser);
                        break;
                    case bc_minecraft_bedrock_project_1.PackType.skin_pack:
                        out = SkinPack_1.SkinPack.Process(diagnoser);
                        break;
                    case bc_minecraft_bedrock_project_1.PackType.world:
                        out = WorldPack_1.WorldPack.Process(diagnoser);
                        break;
                }
            }
        }
        catch (err) {
            let msg;
            if (err.message && err.stack) {
                msg = `${err.message}\n\t${err.stack}`;
            }
            else {
                msg = JSON.stringify(err);
            }
            diagnoser.add({ character: 0, line: 0 }, msg, Severity_1.DiagnosticSeverity.error, "debugger.internal.exception");
        }
        diagnoser.done();
        return out;
    }
    /**Diagnoses the entire given folder
     * @param folder The folder to retrieve files of
     * @param ignores The pattern to ignore on files*/
    processFolder(folder, ignores) {
        const files = this.context.getFiles(folder, ["*"], ignores);
        let out = false;
        for (let I = 0; I < files.length; I++) {
            out = this.process(files[I]) || out;
        }
        return out;
    }
    /**Process the entire given pack
     * @param pack The pack to process
     * @returns True or false is something was processed*/
    processPack(pack) {
        return this.processFolder(pack.folder, pack.context.ignores);
    }
}
exports.Diagnoser = Diagnoser;
//# sourceMappingURL=Diagnoser.js.map